﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.Json.Serialization;
using System.Threading.Tasks;
using RestSharp;

namespace Tr.Sdk
{
    public interface ITrClient
    {
        /// <summary>
        /// 执行操作呼叫中心请求
        /// </summary>
        /// <typeparam name="TRequest"></typeparam>
        /// <typeparam name="TResponse"></typeparam>
        /// <param name="request"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        Task<TResponse?> ExecuteAsync<TRequest, TResponse>(TRequest request, CancellationToken cancellationToken)
            where TRequest : TrRequest;

        Task<string> GetTokenAsync(TokenRequest request, CancellationToken cancellationToken);
    }

    public class TokenRequest : ITrRequest
    {
        public TokenRequest()
        {
            Path = "login/token";
            Method = Method.Get;
        }

        public string Path { get; init; }

        public Method Method { get; init; }

        public string Username { get; set; }

        public string Password { get; set; }
    }

    public interface ITrRequest
    {
        [JsonIgnore]
        string Path { get; }

        [JsonIgnore]
        Method Method { get; }
    }

    public abstract class TrRequest
    {
        public abstract string Path();
        public abstract Method Method();
    }
}
