﻿using System.Net.Http.Json;
using System.Text.Json;
using RestSharp;
using RestSharp.Authenticators;

namespace Tr.Sdk;

public partial class TrClient : ITrClient, IDisposable
{
    private readonly RestClient _client;

    public TrClient(string baseUrl, string apiKey, string apiSecret)
    {
        var options = new RestClientOptions(baseUrl)
        {
            Authenticator = new TrAuthenticator()
        };
        _client = new RestClient(options);
    }

    /// <summary>
    /// 执行操作呼叫中心请求
    /// </summary>
    /// <typeparam name="TRequest"></typeparam>
    /// <typeparam name="TResponse"></typeparam>
    /// <param name="request"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    public async Task<TResponse?> ExecuteAsync<TRequest, TResponse>(TRequest request, CancellationToken cancellationToken)
        where TRequest : TrRequest
    {
        var req = new RestRequest(request.Path(), request.Method())
            .AddObject(request);

        var response = await _client.ExecuteAsync<TResponse>(req, cancellationToken);
        if (!response.IsSuccessful)
        {
            throw new HttpRequestException($"请求呼叫中心服务失败, HttpCode: {response.StatusCode}, Error: {response.ErrorMessage}");
        }
        return response.Data;
    }

    public async Task<string> GetTokenAsync(TokenRequest request, CancellationToken cancellationToken)
    {
        throw new NotImplementedException();
    }

    /// <summary>Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.</summary>
    public void Dispose()
    {
        _client?.Dispose();
        GC.SuppressFinalize(this);
    }
}

public class TrAuthenticator : AuthenticatorBase
{
    public TrAuthenticator() : base("")
    {
    }

    protected override async ValueTask<Parameter> GetAuthenticationParameter(string accessToken)
    {
        Token = string.IsNullOrEmpty(Token) ? "8352d42165d7cf6821e53e926d66adba54c309e6" : Token;
        return new HeaderParameter("X-Auth-Token", Token);
    }
}